﻿-- Author      : RisM
-- Create Date : 9/25/2009 3:02:13 AM


local SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L = LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)

SpeakinSpell:PrintLoading("gui/import.lua")

-------------------------------------------------------------------------------
-- Import New Speeches GUI LAYOUT
-------------------------------------------------------------------------------

SpeakinSpell.OptionsGUI.args.ImportGUI = {
	order = 5,
	type = "group",
	name = "ImportGUI",
	desc = L["Import New Data"],
	args = {
		Header = {
			order = 1,
			type = "header",
			--name = L["Import New Data"].."\n",
			name = function()
				SpeakinSpell:SetLastPageViewed( "ImportGUI", L["Import New Data"] )
				return L["Import New Data"]
			end,
		}, --L["Import New Data"]
		Caption = {
			order = 2,
			type = "description",
			name = function()
				if SpeakinSpell:IsTableEmpty( SpeakinSpell.DEFAULTS.Templates ) then
					return "\n"..L["No New Data to Import"].."\n"
				else
					return L["The following Templates each contain new data"].."\n"
				end
			end,
		},
		AllTemplates = { -- "All Templates"
			order = 3,
			type = "group",
			guiInline = true,
			name = L["All Templates"],
			hidden = function() return not SpeakinSpell:IsTableEmpty( SpeakinSpell.DEFAULTS.Templates ) end,
			args = {
				Description = {
					order = 1,
					type = "description",
					name = L["This group includes All Templates\n"],
				}, --L["This group includes All Templates\n"],
				ShowImportProgress = {
					order = 2,
					type = "toggle",
					name = L["Show Import Progress"],
					desc = L["Report Import Processing Results in the Chat Frame"],
					get = function() return SpeakinSpellSavedData.ShowImportProgress end,
					set = function(_, value) SpeakinSpellSavedData.ShowImportProgress = value end,
				}, -- [X] L["Show Import Progress"]
				GetAll = {
					order = 2,
					type = "execute",
					name = L["Import All Templates"],
					desc = L["Import All Content from All Templates"],
					func = function() SpeakinSpell:ImportGUI_ImportAllTemplates() end,
				}, --L["Import All Templates"],
				SelectTemplate = {
					order = 4,
					type = "select",
					name = L["Select a Template"],
					desc = L["Each of these templates contains new content you might enjoy using in SpeakinSpell"],
					values = function()
						local TemplateList = {}
						for key,Template in pairs( SpeakinSpell.DEFAULTS.Templates ) do
							TemplateList[key] = Template.name
						end
						return TemplateList
					end,
					get = function() return  SpeakinSpell:ImportGUI_OnTemplateSelect("GET",nil) end,
					set = function(_, value) SpeakinSpell:ImportGUI_OnTemplateSelect("SET",value) end,
				}, --L["Select a Template"]
				SelectedTemplate = { -- i.e. "MagePortals (Alliance)"
					order = 5,
					type = "group",
					guiInline = true,
					name = function() return SpeakinSpell:ImportGUI_GetSelectedTemplateName() end,
					args = {
						Description = {
							order = 1,
							type = "description",
							name = function() return SpeakinSpell:ImportGUI_GetSelectedTemplateDesc().."\n" end,
						}, -- i.e. "Mage Portal Announcements for Alliance Cities"
						GetAll = {
							order = 2,
							type = "execute",
							name = L["Import Whole Template"],
							desc = L["Import All Content for the Selected Template"],
							func = function() self:ImportGUI_ImportOneSelectedTemplate() end,
						}, --L["Import Whole Template"]
						Gap = {
							order = 3,
							type = "description",
							name = "\n",
						},
						EventsGroup = { --L["Import Events"]
							order = 4,
							type = "group",
							guiInline = true,
							name = L["Import Events"],
							hidden = function() return not SpeakinSpell:ImportGUI_ShowSelectedTemplateEvents() end,
							args = {
								Description = {
									order = 1,
									type = "description",
									name = function() return SpeakinSpell:ImportGUI_GetSelectedTemplateDesc().."\n" end,
								}, -- i.e. "Mage Portal Announcements for Alliance Cities"
								GetAll = {
									order = 2,
									type = "execute",
									name = L["Import All Events"],
									desc = L["Import All Speech Events for the Selected Template"],
									func = function() self:ImportGUI_ImportAllEvents() end,
								}, --L["Import All Speech Events for the Selected Template"]
								Gap = {
									order = 3,
									type = "description",
									name = "\n",
								},
								EventSelect = {
									order = 4,
									type = "select",
									width = "full",
									name = L["Select a Speech Event"],
									desc = L["Select a Spell TOOLTIP"],
									values = function()
										local EventList = {}
										local EventTable = self:ImportGUI_GetSelectedEventTable()
										if not EventTable then
											return EventTable
										end
										for key,DetectedEvent in pairs( EventTable ) do
											EventList[key] = self:FormatDisplayName( DetectedEvent )
										end
										return EventList
									end,
									get = function() return  SpeakinSpell:ImportGUI_OnEventSelect("GET",nil) end,
									set = function(_, value) SpeakinSpell:ImportGUI_OnEventSelect("SET",value) end,
								}, --L["Select a Speech Event"]
								GetAll = {
									order = 3,
									type = "execute",
									name = L["Import All Speeches"],
									desc = L["Import All speeches for the Selected Speech Event"],
									func = function() self:ImportGUI_ImportAllSpeeches() end,
								}, -- L["Import All speeches for the Selected Speech Event"]
								-- CreateGUI_AddSpeechBoxes will add more controls per speech
								-- local args = SpeakinSpell.OptionsGUI.args.ImportGUI.args.AllTemplates.args.SelectedTemplate.args.EventsGroup.args
								-- args["Speech%02d"] = { group }
							},
						}, -- L["Import Events"]
						RandomSubsGroup = { --L["Import Random Substitutions"]
							order = 5,
							type = "group",
							guiInline = true,
							name = L["Import Random Substitutions"],
							hidden = function() return not SpeakinSpell:ImportGUI_ShowSelectedTemplateRandomSubs() end,
							args = {
								Description = {
									order = 1,
									type = "description",
									name = function() return SpeakinSpell:ImportGUI_GetSelectedTemplateDesc().."\n" end,
								}, -- i.e. "All <randomsubs> for MagePortals (Alliance)"
								GetAll = {
									order = 2,
									type = "execute",
									name = L["Import All <randomsubs>"],
									desc = L["Import All <randomsubs> for the Selected Template"],
									func = function() self:ImportGUI_ImportAllRandomSubs() end,
								}, --L["Import All <randomsubs>"]
								Gap = {
									order = 3,
									type = "description",
									name = "\n",
								},
								RandomSubSelect = {
									order = 3,
									type = "select",
									--width = "full",
									name = L["Select a <substitution>"],
									desc = L["Select a Randomized Substitution\n\nThese substitutions may be used in speeches when announcing events, and a random result from the list below will be selected."],
									values = function()
										local values = {}
										local RandomSubs = self:ImportGUI_GetSelectedRandomSubs()
										if not RandomSubs then
											return values
										end
										for key,_ in pairs( RandomSubs ) do
											values[key] = "<"..key..">"
										end
										return values
									end,
									get = function() return  SpeakinSpell:ImportGUI_OnRandomSubSelect("GET",nil) end,
									set = function(_, value) SpeakinSpell:ImportGUI_OnRandomSubSelect("SET",value) end,
								}, --L["Select a <substitution>"]
								Gap = {
									order = 5,
									type = "description",
									name = "",
									width = "full",
								},
								GetAllWords = {
									order = 6,
									type = "execute",
									name = L["Import All Words"],
									desc = L["Import All \"Words\" for the Selected <randomsub>"],
									func = function() self:ImportGUI_ImportAllRandomWords() end,
								}, --L["Import All Words"]
								-- CreateGUI_AddRandomWordBoxes will add more controls per speech
								-- local args = SpeakinSpell.OptionsGUI.args.ImportGUI.args.AllTemplates.args.SelectedTemplate.args.RandomSubsGroup.args
								-- args["Word%02d"] = { group }
							},
						}, -- L["Import Random Substitutions"]
					},
				},  -- i.e. "MagePortals (Alliance)"
			},
		},
	},
} -- end SpeakinSpell.OptionsGUI.args.ImportGUI
			

-------------------------------------------------------------------------------
-- Validate Selection
-------------------------------------------------------------------------------


function SpeakinSpell:ImportGUI_ValidateSelectionInTable( sel, Table )
	if sel and Table[ sel ] then
		return sel
	end
	for sel,_ in pairs( DTable ) do
		return sel
	end
	return nil
end


function SpeakinSpell:ImportGUI_ValidateSelectedTemplate()
	self.RuntimeData.OptionsGUIStates.ImportGUI.SelectedTemplate = 
		self:ImportGUI_ValidateSelectionInTable(
			self.RuntimeData.OptionsGUIStates.ImportGUI.SelectedTemplate,
			SpeakinSpell.DEFAULTS.Templates )
end


function SpeakinSpell:ImportGUI_ValidateSelectedEvent()
	self.RuntimeData.OptionsGUIStates.ImportGUI.SelectedEvent = 
		self:ImportGUI_ValidateSelectionInTable(
			self.RuntimeData.OptionsGUIStates.ImportGUI.SelectedEvent,
			self:ImportGUI_GetSelectedEventTable() )
end


function SpeakinSpell:ImportGUI_ValidateSelectedRandomSubs()
	self.RuntimeData.OptionsGUIStates.ImportGUI.SelectedRandomSub = 
		self:ImportGUI_ValidateSelectionInTable(
			self.RuntimeData.OptionsGUIStates.ImportGUI.SelectedRandomSub,
			self:ImportGUI_GetSelectedRandomSubs() )
end


-------------------------------------------------------------------------------
-- Safely Get Objects
-------------------------------------------------------------------------------


function SpeakinSpell:ImportGUI_GetSelectedTemplate()
	self:ImportGUI_ValidateSelectedTemplate()

	local sel = self.RuntimeData.OptionsGUIStates.ImportGUI.SelectedTemplate
	if sel then
		return SpeakinSpell.DEFAULTS.Templates[ sel ]
	else
		return nil
	end
end


function SpeakinSpell:ImportGUI_GetSelectedTemplateName()
	local Template = self:ImportGUI_GetSelectedTemplate()
	if Template then
		return Template.name
	else
		return ""
	end
end


function SpeakinSpell:ImportGUI_GetSelectedTemplateDesc()
	local Template = self:ImportGUI_GetSelectedTemplate()
	if Template then
		return Template.desc
	else
		return ""
	end
end


function SpeakinSpell:ImportGUI_GetSelectedEventTable()
	local Template = self:ImportGUI_GetSelectedTemplate()
	if Template and
	   Template.Content then
		return Template.Content.EventTable
	else
		return nil
	end
end


function SpeakinSpell:ImportGUI_GetSelectedRandomSubs()
	local Template = self:ImportGUI_GetSelectedTemplate()
	if Template and
	   Template.Content then
		return Template.Content.RandomSubs
	else
		return nil
	end
end


function SpeakinSpell:ImportGUI_GetSelectedMessages()
	local EventTable = self:ImportGUI_GetSelectedEventTable()
	if EventTable then
		return EventTable.Messages
	else
		return nil
	end
end


function SpeakinSpell:ImportGUI_GetSelectedEventKey()
	self:ImportGUI_ValidateSelectedEvent()
	return self.RuntimeData.OptionsGUIStates.ImportGUI.SelectedEvent
end


function SpeakinSpell:ImportGUI_GetSelectedSubstitutionKey()
	self:ImportGUI_ValidateSelectedRandomSubs()
	return self.RuntimeData.OptionsGUIStates.ImportGUI.SelectedRandomSub
end


-------------------------------------------------------------------------------
-- ImportGUI - Check for Empty Tables / Hidden Controls
-------------------------------------------------------------------------------


function SpeakinSpell:ImportGUI_ShowSelectedTemplateEvents()
	local Template = self:ImportGUI_GetSelectedTemplate()
	if Template then
		return not self:IsTableEmpty( Template.Content.EventTable )
	else
		return ""
	end
end


function SpeakinSpell:ImportGUI_ShowSelectedTemplateRandomSubs()
	local Template = self:ImportGUI_GetSelectedTemplate()
	if Template then
		return not self:IsTableEmpty( Template.Content.RandomSubs )
	else
		return ""
	end
end


-------------------------------------------------------------------------------
-- ImportGUI - EventsGroup
-------------------------------------------------------------------------------


function SpeakinSpell:ImportGUI_GetSpeech( index )
	local Messages = self:ImportGUI_GetSelectedMessages()
	if Messages and
	   Messages[ index ] then
		return Messages[ index ]
	else
		return ""
	end
end


function SpeakinSpell:ImportGUI_ImportSpeech( index )
	local Messages = self:ImportGUI_GetSelectedMessages()
	if Messages and
	   Messages[ index ] then
		local key = self:ImportGUI_GetSelectedEventKey()
		self:ImportTemplate_AddOneSpeech( Template, key, index )
	end
end


function SpeakinSpell:ImportGUI_DeleteSpeech( index )
	local Template = self:ImportGUI_GetSelectedTemplate()
	if Template then
		-- if that was the last new speech in the template, delete it
		local key = self:ImportGUI_GetSelectedEventKey()
		self:Template_DeleteOneSpeech( Template, key, index )
	end
end


-------------------------------------------------------------------------------
-- ImportGUI - RandomSubsGroup
-------------------------------------------------------------------------------

function SpeakinSpell:ImportGUI_GetRandomWord( index )
	local RandomSubs = self:ImportGUI_GetSelectedRandomSubs()
	if RandomSubs and
	   RandomSubs[ index ] then
		return RandomSubs[ index ]
	else
		return ""
	end
end


function SpeakinSpell:ImportGUI_ImportRandomWord( index )
	local RandomSubs = self:ImportGUI_GetSelectedRandomSubs()
	if RandomSubs and
	   RandomSubs[ index ] then
		local key = self:ImportGUI_GetSelectedSubstitutionKey()
		self:ImportTemplate_AddOneRandomWord( Template, key, index )
	end
end


function SpeakinSpell:ImportGUI_DeleteRandomWord( index )
	local Template = self:ImportGUI_GetSelectedTemplate()
	if Template then
		-- also delete empty parent tables
		local key = self:ImportGUI_GetSelectedSubstitutionKey()
		self:Template_DeleteOneRandomWord( Template, key, index )
	end
end


-------------------------------------------------------------------------------
-- Button Handlers
-------------------------------------------------------------------------------


--name = L["Import All Templates"],
--desc = L["Import All Content for All Templates"],
function SpeakinSpell:ImportGUI_ImportAllTemplates()
	for index,Template in pairs( SpeakinSpell.DEFAULTS.Templates ) do
		self:ImportTemplate( Template, index )
	end
end


--name = L["Import Whole Template"],
--desc = L["Import All Content for the Selected Template"],
function SpeakinSpell:ImportGUI_ImportOneSelectedTemplate()
	local Template = self:ImportGUI_GetSelectedTemplate()
	self:ImportTemplate( Template, self.RuntimeData.OptionsGUIStates.ImportGUI.SelectedTemplate )
end


--name = L["Import All Events"],
--desc = L["Import All Speech Events for the Selected Template"],
function SpeakinSpell:ImportGUI_ImportAllEvents()
	local Template = self:ImportGUI_GetSelectedTemplate()
	self:ImportTemplate_Events( Template )
end


--name = L["Import All Speeches"],
--desc = L["Import All speeches for the Selected Speech Event"],
function SpeakinSpell:ImportGUI_ImportAllSpeeches()
	local Template = self:ImportGUI_GetSelectedTemplate()
	local key = self:ImportGUI_GetSelectedEventKey()
	self:ImportTemplate_AllSpeeches( Template, key )
end


--name = L["Import All <randomsubs>"],
--desc = L["Import All <randomsubs> for the Selected Template"],
function SpeakinSpell:ImportGUI_ImportAllRandomSubs()
	local Template = self:ImportGUI_GetSelectedTemplate()
	self:ImportTemplate_RandomSubs( Template )
end


--name = L["Import All Words"],
--desc = L["Import All \"Words\" for the Selected <randomsub>"],
function SpeakinSpell:ImportGUI_ImportAllRandomWords()
	local Template = self:ImportGUI_GetSelectedTemplate()
	local key = self:ImportGUI_GetSelectedSubstitutionKey()
	self:ImportTemplate_AllRandomWords( Template, key )
end


-------------------------------------------------------------------------------
-- OnSelect handlers
-------------------------------------------------------------------------------


function SpeakinSpell:ImportGUI_OnTemplateSelect( getset, value )
	local ImportGUIState = self.RuntimeData.OptionsGUIStates.ImportGUI
	if "GET" == getset then
		self:ImportGUI_ValidateSelectedTemplate()
		return ImportGUIState.SelectedTemplate
	else
		ImportGUIState.SelectedTemplate = value
	end
end


function SpeakinSpell:ImportGUI_OnEventSelect( getset, value )
	local ImportGUIState = self.RuntimeData.OptionsGUIStates.ImportGUI
	if "GET" == getset then
		self:ImportGUI_ValidateSelectedEvent()
		return ImportGUIState.SelectedEvent
	else
		ImportGUIState.SelectedEvent = value
	end
end


function SpeakinSpell:ImportGUI_OnRandomSubSelect( getset, value)
	local ImportGUIState = self.RuntimeData.OptionsGUIStates.ImportGUI
	if "GET" == getset then
		self:ImportGUI_ValidateSelectedRandomSub()
		return ImportGUIState.SelectedRandomSub
	else
		ImportGUIState.SelectedRandomSub = value
	end
end


-------------------------------------------------------------------------------
-- CreateGUI - Import Templates GUI
-------------------------------------------------------------------------------


function SpeakinSpell:CreateGUI_AddSpeechBoxes()
	local funcname = "CreateGUI_AddSpeechBoxes"
	
	-- Add Random Speech boxes
	local args = SpeakinSpell.OptionsGUI.args.ImportGUI.args.AllTemplates.args.SelectedTemplate.args.EventsGroup.args
	for index=1,SpeakinSpell.MAX.MESSAGES_PER_SPELL,1 do
		local frameName = string.format( "Speech%02d", index )
		args[frameName] = { -- "Random Speech 1" (group)
			order = index + 100,
			type = "group",
			name = string.format( L["Random Speech %d"], index ),
			args = {
				Speech = {
					order = 1,
					type = "description",
					name = function() self:ImportGUI_GetSpeech( index ) end,
				}, -- "Foo",
				Use = {
					order = 2,
					type = "execute",
					name = SpeakinSpell.Colors.ON..L["Use"],
					desc = L["Use this Random Speech for the selected Event"],
					func = function() SpeakinSpell:ImportGUI_ImportSpeech( index ) end,
				}, -- "Use This Speech",
				Delete = {
					order = 3,
					type = "execute",
					name = SpeakinSpell.Colors.OFF..L["Delete"],
					desc = string.format( L["%sDon't|r Use this Random Speech for the selected Event"], SpeakinSpell.Colors.OFF ),
					func = function() SpeakinSpell:ImportGUI_DeleteSpeech( index ) end,
				}, -- "Delete This Speech",
			},
		}
	end
end


function SpeakinSpell:CreateGUI_AddRandomWordBoxes()
	-- add more controls per random word input box (group)
	local args = SpeakinSpell.OptionsGUI.args.ImportGUI.args.AllTemplates.args.SelectedTemplate.args.RandomSubsGroup.args
	for index=1,SpeakinSpell.MAX.TEXTS_PER_RANDOM_SUB,1 do
		local frameName = string.format( "Word%02d", index )
		args[frameName] = { -- "Random Word 1" (group)
			order = index + 100,
			type = "group",
			name = string.format( L["Random Word %d"], index ),
			args = {
				Word = {
					order = 1,
					type = "description",
					name = function() self:ImportGUI_GetRandomWord( index ) end,
				}, -- "Foo",
				Use = {
					order = 2,
					type = "execute",
					name = SpeakinSpell.Colors.ON..L["Use"],
					desc = L["Use this Random Word for the selected <substitution>"],
					func = function() SpeakinSpell:ImportGUI_ImportRandomWord( index ) end,
				}, -- "Use This Word",
				Delete = {
					order = 3,
					type = "execute",
					name = SpeakinSpell.Colors.OFF..L["Delete"],
					desc = string.format( L["%sDon't|r Use this Random Word for the selected <substitution>"], SpeakinSpell.Colors.OFF ),
					func = function() SpeakinSpell:ImportGUI_DeleteRandomWord( index ) end,
				}, -- "Delete This Word",
			},
		}
	end
end


-------------------------------------------------------------------------------
-- CreateGUI - ImportGUI Main CreateGUI Entry
-------------------------------------------------------------------------------


function SpeakinSpell:CreateGUI_Import()
	self:CreateGUI_AddSpeechBoxes()
	self:CreateGUI_AddRandomWordBoxes()
end

